----------------------------------------------------------------------------------
-- Create Date:    10:14:44 02/06/2020 
-- Design Name: 
-- Module Name:    UART - Behavioral 
-- Project Name: 
----------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY uart IS
PORT (
  CLK:       IN  STD_LOGIC;
  CLR:       IN  STD_LOGIC;
  A0:        IN  STD_LOGIC;    
  CE:        IN  STD_LOGIC;  
  WE:        IN  STD_LOGIC;
  DATA_IN:   IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
  DATA_OUT:  OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
  RX:        IN  STD_LOGIC;
  TX:        OUT STD_LOGIC;
  TP:        OUT STD_LOGIC );
END uart;

ARCHITECTURE  uart_arch OF uart IS

	COMPONENT serial_port
	PORT (
	  clk:       IN  STD_LOGIC;
	  clr:       IN  STD_LOGIC; 
	  wr:        IN  STD_LOGIC;
	  data_i:    IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
	  data_o:    OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
	  rx:        IN  STD_LOGIC;
	  tx:        OUT STD_LOGIC;
	  rx_valid:  OUT STD_LOGIC;
	  rx_idle:   OUT STD_LOGIC;
	  tx_idle:   OUT STD_LOGIC;
	  baud_test: OUT STD_LOGIC );
	END COMPONENT;
	
	SIGNAL en         : STD_LOGIC;
	SIGNAL data_o     : STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL status_o   : STD_LOGIC_VECTOR(7 DOWNTO 0);	
	
	SIGNAL rx_valid   : STD_LOGIC;
	SIGNAL rx_idle    : STD_LOGIC;
	SIGNAL tx_idle    : STD_LOGIC;
	
	SIGNAL data_valid : STD_LOGIC;
	SIGNAL data_read  : STD_LOGIC;
	
   SIGNAL test	      : STD_LOGIC;
	
BEGIN

  TP <= data_valid;

  en <= ce AND we;
  data_read <= ce AND (NOT we) AND (A0);
  status_o <= "00000" & tx_idle & rx_idle & data_valid;
  
  DATA_OUT(15 DOWNTO 8) <= "00000000";
  
  MUX : PROCESS( A0, data_o, status_o )
  BEGIN
    IF A0='0'
	 THEN
	   DATA_OUT(7 DOWNTO 0) <=  status_o;
    ELSE
	   DATA_OUT(7 DOWNTO 0) <=  data_o;
    END IF;
  END PROCESS;	  

  RX_FLAG : PROCESS( clk, clr )
  BEGIN
    IF clr='1'
	 THEN
	   data_valid <= '0';
    ELSIF clk='1' and clk'event
    THEN
      IF rx_valid='1'
      THEN
    	  data_valid <= '1';
      ELSIF data_read='1'	
      THEN
    	  data_valid <= '0';	
      END IF;
    END IF;
  END PROCESS;

  comms : serial_port PORT MAP(
    clk       => CLK,
	 clr       => CLR,
	 wr        => en,    
	 data_i    => DATA_IN(7 DOWNTO 0),
	 data_o    => data_o,
	 rx        => RX,
	 tx        => TX,
	 rx_valid  => rx_valid,
	 rx_idle   => rx_idle,
	 tx_idle   => tx_idle,
	 baud_test => test );

end uart_arch;
